##############################################################################
##
## ***************************************************************************
## **                                                                       **
## ** Copyright (c) 1995-2004 Xilinx, Inc.  All rights reserved.            **
## **                                                                       **
## ** You may copy and modify these files for your own internal use solely  **
## ** with Xilinx programmable logic devices and Xilinx EDK system or       **
## ** create IP modules solely for Xilinx programmable logic devices and    **
## ** Xilinx EDK system. No rights are granted to distribute any files      **
## ** unless they are distributed in Xilinx programmable logic devices.     **
## **                                                                       **
## ***************************************************************************
##
##############################################################################
## Filename:          C:\james\audio_capture\drivers\fsl_new_core_v1_00_a\data\fsl_new_core_v2_1_0.tcl
## Description:       Tool Command Language
## Date:              Mon Mar 21 20:42:20 2005 (by Custom Function Wizard)
##############################################################################

## Note:
## This tcl file will detect the FSL id number of the connected FSL interface,
## and generate the right header file.

proc open_include_file {file_name} {
    set filename [file join "../../include/" $file_name]
    if {[file exists $filename]} {
        set config_inc [open $filename a]
    } else {
        set config_inc [open $filename a]
        xprint_generated_header $config_inc "MP3 Parameters"
    }
    return $config_inc
}

proc generate {drv_handle} {
   puts "Generating Macros for FSL peripheral access ..."
    set hw_inst_handle [xget_sw_parameter_handle $drv_handle "HW_INSTANCE"] 
    set hw_inst [xget_value $hw_inst_handle "value"]
    set drv_name_handle [xget_sw_parameter_handle $drv_handle "DRIVER_NAME"]
    set ipname [xget_value $drv_name_handle "value"]
    set header_file [format "x%s_fsl.h" $ipname]
    fsl_defines $header_file $hw_inst $ipname
}

proc fsl_defines {file_name core_name ipname} {
    set core_def_name [string toupper $ipname]
    if {[string compare -nocase "none" $core_name] != 0} {
	set conffile  [open_include_file $file_name]

	set sw_prochandle [xget_libgen_proc_handle]
	set ip_handle [xget_sw_ipinst_handle_from_processor $sw_prochandle $core_name]
	set sfsl_name [xget_value $ip_handle "BUS_INTERFACE" "SFSL"]
	set mfsl_name [xget_value $ip_handle "BUS_INTERFACE" "MFSL"]

	set mhs_handle [xget_handle $ip_handle "parent"]

	set sfsl_master [xget_hw_connected_busifs_handle $mhs_handle $sfsl_name "master"]
	set sfsl_index [xget_value $sfsl_master "NAME"]
	set sfsl_index [string toupper $sfsl_index]
	set sfsl_index [string map {MFSL ""} $sfsl_index]

	set mfsl_slave [xget_hw_connected_busifs_handle $mhs_handle $mfsl_name "slave"]
	set mfsl_index [xget_value $mfsl_slave "NAME"]
	set mfsl_index [string toupper $mfsl_index]
	set mfsl_index [string map {SFSL ""} $mfsl_index]

	puts  $conffile "static volatile int rfsl0, rfsl1, rfsl2, rfsl3, rfsl4, rfsl5, rfsl6, rfsl7;"

	puts  $conffile "\#define READ_${core_def_name}(val)  asm(\"get \%0, \%1\" : \"=d\" (\#\#val\#\#) : \"m\" (rfsl${sfsl_index}))"
	puts  $conffile "\#define WRITE_${core_def_name}(val) asm(\"put \%1, \%0\" : \"=m\" (rfsl${mfsl_index}) : \"d\" (\#\#val\#\#))"
	puts  $conffile "\#define WRITE_CNTL_${core_def_name}(val) asm(\"cput \%1, \%0\" : \"=m\" (rfsl${mfsl_index}) : \"d\" (\#\#val\#\#))"
	puts $conffile ""
	close $conffile
    } 
}

