------------------------------------------------------------------------------
-- audiofft1024 - entity/architecture pair
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- ** Copyright (c) 1995-2004 Xilinx, Inc.  All rights reserved.            **
-- **                                                                       **
-- ** Xilinx, Inc.                                                          **
-- ** XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"         **
-- ** AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND       **
-- ** SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,        **
-- ** OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,        **
-- ** APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION           **
-- ** THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,     **
-- ** AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE      **
-- ** FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY              **
-- ** WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE               **
-- ** IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR        **
-- ** REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF       **
-- ** INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       **
-- ** FOR A PARTICULAR PURPOSE.                                             **
-- **                                                                       **
-- ** YOU MAY COPY AND MODIFY THESE FILES FOR YOUR OWN INTERNAL USE SOLELY  **
-- ** WITH XILINX PROGRAMMABLE LOGIC DEVICES AND XILINX EDK SYSTEM OR       **
-- ** CREATE IP MODULES SOLELY FOR XILINX PROGRAMMABLE LOGIC DEVICES AND    **
-- ** XILINX EDK SYSTEM. NO RIGHTS ARE GRANTED TO DISTRIBUTE ANY FILES      **
-- ** UNLESS THEY ARE DISTRIBUTED IN XILINX PROGRAMMABLE LOGIC DEVICES.     **
-- **                                                                       **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:          audiofft1024
-- Version:           1.00.a
-- Description:       FSL core wrapper.
-- Date:              Mon Mar 21 22:21:17 2005 (by Custom Function Wizard)
-- VHDL-Standard:     VHDL'93
------------------------------------------------------------------------------
-- Naming Conventions:
-- 	active low signals:                    "*_n"
-- 	clock signals:                         "clk", "clk_div#", "clk_#x"
-- 	reset signals:                         "rst", "rst_n"
-- 	generics:                              "C_*"
-- 	user defined types:                    "*_TYPE"
-- 	state machine next state:              "*_ns"
-- 	state machine current state:           "*_cs"
-- 	combinatorial signals:                 "*_com"
-- 	pipelined or register delay signals:   "*_d#"
-- 	counter signals:                       "*cnt*"
-- 	clock enable signals:                  "*_ce"
-- 	internal version of output port:       "*_i"
-- 	device pins:                           "*_pin"
-- 	ports:                                 "- Names begin with Uppercase"
-- 	processes:                             "*_PROCESS"
-- 	component instantiations:              "<ENTITY_>I_<#|FUNC>"
------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_arith.all;         
use ieee.std_logic_signed.all;
--use ieee.std_logic_unsigned.all;      


------------------------------------------------------------------------------
--
-- Definition of Parameters
-- C_WIDTH            : Signal width
--
-- C_INPUT_FSL_DEPTH	 : Input FSL bus depth
-- C_OUTPUT_FSL_DEPTH	: Output FSL bus depth
--
-- Definition of Ports
-- CLK             : Synchronous clock
-- RESET           : System reset, should always come from FSL bus
-- FSL_S_CLK       : Slave asynchronous clock
-- FSL_S_READ      : Read signal, requiring next available input to be read
-- FSL_S_DATA      : Input data
-- FSL_S_CONTROL   : Control Bit, indicating the input data are control word
-- FSL_S_EXISTS    : Data Exist Bit, indicating data exist in the input FSL bus
-- FSL_M_CLK       : Master asynchronous clock
-- FSL_M_WRITE     : Write signal, enabling writing to output FSL bus
-- FSL_M_DATA      : Output data
-- FSL_M_CONTROL   : Control Bit, indicating the output data are contol word
-- FSL_M_FULL      : Full Bit, indicating output FSL bus is full
--
-------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Entity Section
--------------------------------------------------------------------------------

entity audiofft1024 is
	
	generic (
		C_DWIDTH        : integer := 16;
		
		-- CAUTION:
		--
		-- These two parameters determine the default value of
		-- C_FSL_DEPTH for FSL bus.
		--
		-- Do not change the names of these parameters.
		--
		-- The default depths of the input and output FSL buses
		-- are set to the total number of input and output
		-- words respectively.
		--
		-- If you change the default values here, remember to update
		-- them in fft_fsl_v2_1_0.mpd file.
		C_INPUT_FSL_DEPTH  : natural := 1024;
		C_OUTPUT_FSL_DEPTH : natural := 1024
	);
	
	port 
	(
		-- Bus protocol ports. 
		CLK	               : in	std_logic;
		RESET	             : in	std_logic;
		FSL_S_CLK	         : out	std_logic;
		FSL_S_READ	        : out	std_logic;
		--FSL_S_DATA	: in	std_logic_vector(0 to 31);
      FSL_S_DATA	        : in	std_logic_vector(0 to C_DWIDTH-1);  --added by sj
		FSL_S_CONTROL	     : in	std_logic;
		FSL_S_EXISTS	      : in	std_logic;
		FSL_M_CLK	         : out	std_logic;
		FSL_M_WRITE	       : out	std_logic;
		FSL_M_DATA	        : out	std_logic_vector(0 to 31);
		--FSL_M_DATA	: out	std_logic_vector(0 to C_DWIDTH-1);   --added by sj
		FSL_M_CONTROL	     : out	std_logic;
		FSL_M_FULL	        : in	std_logic
	);
end audiofft1024;

--------------------------------------------------------------------------------
-- Architecture Section
--------------------------------------------------------------------------------
architecture EXAMPLE of audiofft1024 is

-- import external component
component fft1024radix2 is 
	port (
	xn_re                    : IN std_logic_VECTOR(15 downto 0);
	xn_im                    : IN std_logic_VECTOR(15 downto 0);
	start                    : IN std_logic;
	unload                   : IN std_logic;
	nfft                     : IN std_logic_VECTOR(4 downto 0);
	nfft_we                  : IN std_logic;
	fwd_inv                  : IN std_logic;
	fwd_inv_we               : IN std_logic;
	sclr                     : IN std_logic;
	ce                       : IN std_logic;
	clk                      : IN std_logic;
	xk_re                    : OUT std_logic_VECTOR(26 downto 0);
	xk_im                    : OUT std_logic_VECTOR(26 downto 0);
	xn_index                 : OUT std_logic_VECTOR(9 downto 0);
	xk_index                 : OUT std_logic_VECTOR(9 downto 0);
	rfd                      : OUT std_logic;
	busy                     : OUT std_logic;
	dv                       : OUT std_logic;
	edone                    : OUT std_logic;
	done                     : OUT std_logic);
end component fft1024radix2;

	-- signals connected to fft1024radix2
	signal xn_re             : std_logic_VECTOR(15 downto 0);
	signal xn_im             : std_logic_VECTOR(15 downto 0);
	signal start             : std_logic;
	signal unload            : std_logic;
	signal nfft              : std_logic_VECTOR(4 downto 0);
	signal nfft_we           : std_logic;
	signal fwd_inv           : std_logic;
	signal fwd_inv_we        : std_logic;
	signal sclr              : std_logic;
	signal ce                : std_logic;
	--signal clk               : std_logic; --?
	signal xk_re             : std_logic_VECTOR(26 downto 0);
	signal xk_im             : std_logic_VECTOR(26 downto 0);
	signal xn_index          : std_logic_VECTOR(9 downto 0);
	signal xk_index          : std_logic_VECTOR(9 downto 0);
	signal rfd               : std_logic;
	signal busy              : std_logic;
	signal dv                : std_logic;
	signal edone             : std_logic;
	signal done              : std_logic;
	
	-- states & bookeeping signals
	signal count             : std_logic_vector(4 downto 0);
	signal input_data        : std_logic_vector(15 downto 0);
	signal output_data       : std_logic_vector(15 downto 0);
	signal output_is_valid   : std_logic;
	signal fsl_is_full       : std_logic;
	signal dataInBuf         : std_logic_vector(15 downto 0);
   signal fftDataBuf_re     : std_logic_vector (15 downto 0);
   signal fftDataBuf_im     : std_logic_vector (15 downto 0);
   signal fftMagnitude      : std_logic_vector (31 downto 0); 
   
-- In the Custom Function Wizard you specified the following
-- Number of input arrays: 1
-- Size of each input array: 512
-- This constant contain the total number of input words,
-- which is the product of the above
constant NUMBER_OF_INPUT_WORDS  : natural := 16; --512;

-- In the Custom Function Wizard you specified the following
-- Number of output arrays: 1
-- Size of each output array: 512
-- This constant contain the total number of input words,
-- which is the product of the above
constant NUMBER_OF_OUTPUT_WORDS : natural := 16; --512;

-- Finite States
type STATE_TYPE is (Ready, Idle, Read_Inputs, Write_Outputs, Cal_fft, End_Of_Read,End_Of_Write);

signal state        : STATE_TYPE;

-- Accumulator to hold sum of inputs read at any point in time
signal sum          : std_logic_vector(0 to 31);

-- Counters to store the number inputs read & outputs written
signal nr_of_reads  : natural range 0 to NUMBER_OF_INPUT_WORDS - 1;
signal nr_of_writes : natural range 0 to NUMBER_OF_OUTPUT_WORDS - 1;

begin

-- Include FFT 1024 generated core component
fft1024radix2_I : fft1024radix2
	port map (
	xn_re      => xn_re,       -- : IN std_logic_VECTOR(15 downto 0);
	xn_im      => xn_im,       -- : IN std_logic_VECTOR(15 downto 0);
	start      => start,       -- : IN std_logic;
	unload     => unload,      -- : IN std_logic;
	nfft       => nfft,        -- : IN std_logic_VECTOR(4 downto 0);
	nfft_we    => nfft_we,     -- : IN std_logic;
	fwd_inv    => fwd_inv,     -- : IN std_logic;
	fwd_inv_we => fwd_inv_we,  -- : IN std_logic;
	sclr       => sclr,        -- : IN std_logic;
	ce         => ce,          -- : IN std_logic;
	clk        => clk,         -- : IN std_logic;
	xk_re      => xk_re,       -- : OUT std_logic_VECTOR(15 downto 0);
	xk_im      => xk_im,       -- : OUT std_logic_VECTOR(15 downto 0);
	xn_index   => xn_index,    -- : OUT std_logic_VECTOR(9 downto 0);
	xk_index   => xk_index,    -- : OUT std_logic_VECTOR(9 downto 0);
	rfd        => rfd,         -- : OUT std_logic;
	busy       => busy,        -- : OUT std_logic;
	dv         => dv,          -- : OUT std_logic;
	edone      => edone,       -- : OUT std_logic;
	done       => done);       -- : OUT std_logic;

-- CAUTION:
-- The sequence in which data are read in and written out should be
-- consistant with the sequence they are written and read in the
-- driver's fft_fsl.c file

FSL_S_READ  <= FSL_S_EXISTS   when state = Read_Inputs   else '0';
-- FSL_M_WRITE <= not FSL_M_Full when state = Write_Outputs else '0';

The_SW_accelerator : process (CLK,RESET) is
begin  -- process The_SW_accelerator
    
   if RESET = '1' then                     -- Synchronous reset (active high)
     -- Initialize all parameters on RESET signal
     nr_of_reads  <= 0;            
     nr_of_writes <= 0;
     count        <= "00000";              -- set counter to zero
     ce           <= '1';                  -- set clock enable to '1' 
     sclr         <= '1';                  -- set Master Reset to '1'
     unload       <= '0';                  -- set unload results to '0'
     start        <= '0';                  -- set start signal to '0'
     xn_im        <= "0000000000000000";   -- intialize imaginary component to 0
     state        <= Ready;                -- goto ready state
   elsif CLK'event and CLK = '1' then      -- Rising clock edge
     case state is
       -- READY STATE: Set all require parameters for FFT 
       when Ready =>
         			if count = 0 then    
				   sclr        <= '0';         -- Master Reset to '0'
				 	nfft_we     <= '1';         -- NFFT set to '1' to invoke signal
				 	nfft        <= "00100";     -- Set to 16 point NFFT
				 	count       <= count + 1;   -- Update Clock count
				   FSL_M_WRITE <= '0';         -- set FSL_M_WRITE to '0', no data written to FFT
				elsif count = 1 then        
					nfft_we     <= '0';         
					nfft        <= "00000";      
					fwd_inv_we  <= '1';         -- Setup fwd or inv NFFT
					fwd_inv     <= '1';         -- Set to '1' for forward FFT 
					start       <= '1';         -- Set 'start' signal to high
					count       <= count + 1;   -- update clock count
				
				elsif count = 2 then          
					fwd_inv_we  <= '0';   
					fwd_inv     <= '0';       
					start       <= '0';         -- down 'start' signal
					count       <= count + 1;   -- update clock count
				
				elsif count = 4 then           -- wait for 3 clock cycle before starting to read input    
					count       <= "00000";     -- reset count to zero
			--		ce          <= '0';         -- clock is disabled
					nr_of_reads <= NUMBER_OF_INPUT_WORDS - 1;
					state       <= Idle;        -- goto next state 'Idle'				   
				else
				   count       <= count + 1;   -- otherwise waste clock cycle
 				end if;	
 				     
       -- IDLE STATE: wait until there is an input to read 				     
       when Idle =>                       
         if (FSL_S_EXISTS = '1') then      -- If data exists
           dataInBuf   <= FSL_S_DATA;      -- store to a buffer
           nr_of_reads <= nr_of_reads - 1; -- subtract number of data read 
           ce          <= '1';             -- enable clock
           state       <= Read_Inputs;     -- goto Read_Input state
         else                              -- If data does NOT exists
           ce          <= '0';             -- disable the clock
         end if;

       -- READ_INPUT STATE: Reads input and write to xn_re and xn_im port of FFT
       when Read_Inputs =>                                 
         if (FSL_S_EXISTS = '1') then                        -- If data exists
           xn_re       <= dataInBuf;                         -- write to FFT
           xn_im       <= "0000000000000000";                 
           dataInBuf   <= FSL_S_DATA;                        -- read another input
           
           if (nr_of_reads = 0) then                         -- If all N points are read to FFT 
             nr_of_writes <= NUMBER_OF_OUTPUT_WORDS - 1;
             state        <= End_Of_Read;                    -- goto next state
           else
             nr_of_reads  <= nr_of_reads - 1;                -- If not, continue reading
           end if;
           
         else                                                -- If data does NOT exist
            xn_re  <= dataInBuf;                             -- pass the previously read data to FFT
            xn_im  <= "0000000000000000";                     
            ce     <= '0';                                   -- disable clock
            state  <= Idle;                                  -- Goto Idle state
         end if;
       
       -- END_OF_READ STATE: Wait for FFT to perform calculation  
       when End_Of_Read =>
           xn_re   <= dataInBuf;                             -- Pass the previously read data to FFT 
           xn_im   <= "0000000000000000";         
           state   <= Cal_fft;                               -- Goto CAL_FFT state 
       
       -- CAL_FFT STATE: wait for FFT to perform calculation
       when Cal_fft =>
           ce      <= '1';
           
           if edone = '1' then                                  -- If calculation is complete              
              unload <= '1';                                    -- Invoke unload result
           else
              unload <= '0';                                    -- else continue wait
           end if;
           
           if dv = '1' then                                     -- If valid data is found   
              fftDataBuf_re <= xk_re(26) & xk_re(14 downto 0);  -- read the data
              fftDataBuf_im <= xk_im(26) & xk_im(14 downto 0);
              state         <= Write_Outputs;                   -- then goto 'Write_Outputs' state
           end if;
       
       -- WRITE_OUTPUTS STATE: Write the FFT result to FSL            
       when Write_Outputs =>           
           
           if (nr_of_writes = 0) then                           -- until all results are read     
              state <= End_Of_Write;                            -- goto next state  
           else
              if (FSL_M_FULL = '0') then                        -- while FIFO is not full
                 FSL_M_WRITE   <= '1';                          -- write FFT result to FIFO 
                 nr_of_writes  <= nr_of_writes - 1;
                 
                 -- calculate maginitude of the frequency
                 fftMagnitude  <= fftDataBuf_re*fftDataBuf_re + fftDataBuf_im*fftDataBuf_im;
                 
                 FSL_M_DATA    <= std_logic_vector (fftMagnitude);    -- Write to FIFO  
                 fftDataBuf_re <= xk_re(26) & xk_re(14 downto 0);
                 fftDataBuf_im <= xk_im(26) & xk_im(14 downto 0);
              end if;
           end if;
       
       -- END_OF_WRITE STATE: Completed one bash   
       when End_Of_Write =>
           fftMagnitude  <= fftDataBuf_re*fftDataBuf_re + fftDataBuf_im*fftDataBuf_im;
           FSL_M_DATA    <= std_logic_vector (fftMagnitude);
           FSL_M_WRITE   <= '1';                                -- Write to FSL FIFO
           state         <= Ready;                              -- go back to ready state
       
       -- OTHERWISE: Reset and goto READY State    
       when others =>
           sclr          <= '0';
			  ce            <= '1';
			  nfft          <= "00000";
			  nfft_we       <= '0';
			  fwd_inv       <= '0';
			  fwd_inv_we    <= '0';
			  start         <= '0';
			  unload        <= '0';
			  state         <= Ready;                              -- goto 'Ready' State
     
     end case;
   end if;
   
end process The_SW_accelerator;
-- [End] of process

end architecture EXAMPLE;
-- [end] of Architecture